import sys
import time
import argparse
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d.axes3d import Axes3D
from matplotlib import cm


from tkobject import tkObject

class tkPlot(tkObject):
    def __init__(self, figsize = None, **kwargs):
        self.figure = plt.figure(figsize = figsize)
        self.axes  = []
        self.lines = []
        self.update(**kwargs)

    def initialize(self, **kwargs):
        super().__init__(**kwargs)

#    def __del__(self):
#        print("{} destroyed".format(self.name))

    def __str__(self):
        return "optimization object by {}".format(self.method)


    def pause(self, duration = 0.001, **kwargs):
        plt.pause(duration, **kwargs)

    def show(self, **kwargs):
        plt.show(**kwargs)

    def sleep(self, duration):
        time.sleep(duration)

    def adjust_list(self, n):
        for i in range(len(self.axes), n+1):
            self.axes.append(None)

    def add_subplot(self, nrows = 1, ncols = 1, index = 0, **kwargs):
        """
            add_subplot: heritated from matplot.lib.pyplot.subplot
                The third parameter is counted from zero, not from the original definition from 1.
        """
        
        ax = self.figure.add_subplot(nrows, ncols, index+1, **kwargs)
        try:
            ax.linelist
        except:
            ax.linelist = []
            ax.xlist    = []
            ax.ylist    = []
            ax.zlist    = []
        self.adjust_list(index)
        self.axes[index] = ax
        return ax

    def plot(self, iaxes = 0, xd = None, yd = None,
                color = 'blue', linestyle = '', linewidth = 0.5,
                fillstyle = 'full', marker = '', markersize = 5,
                **kwargs):
        lines = self.axes[iaxes].plot(xd, yd,
                color = color, linestyle = linestyle, linewidth = linewidth,
                fillstyle = fillstyle, marker = marker, markersize = markersize,
                **kwargs)
        self.axes[iaxes].linelist.append(lines)
        self.axes[iaxes].xlist.append(xd)
        self.axes[iaxes].ylist.append(yd)
        self.axes[iaxes].zlist.append([])
        return lines

    def plot_wireframe(self, iaxes = 0, xd = None, yd = None, zd = None,
                rstride = 2, cstride = 2):
        surface = self.axes[iaxes].plot_wireframe(xd, yd, zd, rstride = rstride, cstride = cstride)
        self.axes[iaxes].linelist.append(surface)
        self.axes[iaxes].xlist.append(xd)
        self.axes[iaxes].ylist.append(yd)
        self.axes[iaxes].zlist.append(zd)
        return surface

    def plot_contour(self, iaxes = 0, xd = None, yd = None, zd = None,
                levels = 51, cmap = 'Spectral', **kwargs):
        if cmap == 'line':
            contour = self.axes[iaxes].contour(xd, yd, zd, levels = levels, **kwargs)
        else:
            contour = self.axes[iaxes].contourf(xd, yd, zd, levels = levels, cmap = cmap, **kwargs)
        if kwargs['aspect'] is not None:
            self.axes[iaxes].set_aspect(kwargs['aspect'])
        self.axes[iaxes].linelist.append(contour)
        self.axes[iaxes].xlist.append(xd)
        self.axes[iaxes].ylist.append(yd)
        self.axes[iaxes].zlist.append(zd)
        return contour

    def set_data(self, iaxes, ilines, idx, xt, yt):
        self.axes[iaxes].linelist[ilines][idx].set_data(xt, yt)

    def set_axtitle(self, iaxes, title):
        self.axes[iaxes].set_title(title)
